// Fichier Actions_Barillet.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *              En-tete du module de Barillet           *
 *                                                      *
 * .Date de cration : 11 Fvrier 2008                  *
 * .Auteur : Cardineau Matthieu							*       
 *                                                      *
 * .Descriptions :                                      *
 *      -fonctions pour gerer les actions de la machine *
 * a tats du barillet									*
 * 														*
 *                                                      *
 * .Grandes modifications :                             *
 *  __________________________________________________  *
 * |   DATE   |   Auteur   |          Detail          | *
 * |--------------------------------------------------| *
 * | 11/02/08 | M.Cardineau| Entete ajoute           | *
 * |          |            | -Stocker                 | *
 * |          |            | -Deposer_Bleu/blanc/rouge| *
 * |--------------------------------------------------| *
 * | 13/02/08 | M.Cardineau| -Tir_Bleu/blanc/rouge    | *
 * |--------------------------------------------------| *
 * | 18/02/08 | M.Cardineau| -SlotVideEntree		  | *
 * |--------------------------------------------------| *
 * |          |            |                          | *
 * |--------------------------------------------------| *
 * |__________|____________|__________________________| *
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *														*
 ********************************************************/

//------------------------------DEFINITIONS-------------------------------//
#define BLEU 1						//une balle bleue correspond a un tat 1
#define BLANC 2						//idem pour blanc
#define ROUGE 3						//et pour rouge
//------------------------------------------------------------------------//

//------------------------------INCLUSIONS-------------------------------//
#include <p18f452.h>		//librairie du PIC
#include <delays.h>			//pour les tempos

#include "Servos.h"				// Librairie pour commander les servomoteurs (4maxi)
#include "Barillet.h"			// Librairie pour l'interuption d'asservissement
#include "Capt_Coul.h"			// Librairie pour gerer le capteur de couleur
#include "Actions_Barillet.h"	// Entte de la librairie
#include "Depose_map.h"		// Mapping de dpose des balles
#include "i2c_Slave.h"

//-----------------------------------------------------------------------//

//-----------------------------------VARIABLES GLOBALES---------------------------------------//
extern unsigned char NumSlotReel;						//numro du slot rel
extern unsigned char NumSlotDesire;						//numro du slot demand
extern unsigned char Couleur_Adversaire;				// Couleur joue par l'adversaire

extern unsigned char 			Buffer_Ordre[TAILLE_BUFFER];		// Tableau qui bufferise les ordres  executer
extern unsigned char			Indice_Courant;						// Pour pointer l'ordre courant dans le tableau des ordres
//-------------------------------------------------------------------------------------------//




//fonction pour initialiser le barillet
/*Vide le tableau Slot
 */
unsigned char Barillet_Init(volatile unsigned char *Slot)
{	unsigned char i=0;
	NumSlotReel=0;												// On replace virtuellement le barillet dans sa position initiale
	for (i=0; i<5;i++) Slot[i] = 0; 							// On vide le tableau.
}




//fonction pour vider le barillet
/*depose pour tout les slot
 */
void Vider_Barillet(volatile unsigned char *Slot)
{	
	unsigned char i=0;	
	for (i=0; i<5;i++) 
	{			
		Servo_Depose();
		FermerBarriere();	
		while (AvanceUnSlot ())
		{
			OuvrirBarriere();
			Delay10KTCYx(100);
		}		
	}
	OuvrirBarriere();							
	Barillet_Init(Slot);										// Initialise les variables du barillet 
}




//Fonction pour stocker une balle que l'on detecte.
/* On passe au slot suivant et on stocke la balle dans le tableau
 * Le premier argument est le tableau Slot contenu du barillet
 * Le second argument est la couleur joue par l'averssaire
 * 
 */
void Stocker (volatile unsigned char *Slot, volatile unsigned char Couleur_Adversaire)
{	
	unsigned char CouleurBalle;							// Variable utilise pour mmoriser la couleur de la balle
	unsigned char PosBarillet=NumSlotReel;				// Mmorise la position courante de la balle pour le stockage de la balle
	
	FermerBarriere();									// On ferme la barriere pour empecher les balles d'entrer
	if (AvanceUnSlot()) 								// Si la fonction ne retourne pas 0 alors on a une erreur
	{	
		return ; 										// On quitte la routine de stockage
	}
	
	CouleurBalle = LireCaptCoul();						// Lit la couleur de la balle	
	if (CouleurBalle==Couleur_Adversaire)				// C'est ue balle adverse, on ne la conserve pas
	{
		PlaceBarillet ((NumSlotReel+3)%5);				// Place la balle bleu devant le mcanisme de dpose	
		Servo_Depose();									// Ejecte la balle		
		return;											// On quitte la fonction
	}	
	
	Slot[PosBarillet]=CouleurBalle;						// On mmorise la couleur de la balle
	
}
		
	
	


//fonction pour mettre un slot vide a l'entre
/*Cherche le slot vide le plus proche et positionne le barillet en fonction
 *On ferme la barriere quand on execute cette action
 */
unsigned char SlotVideEntree (volatile unsigned char *Slot)
{	unsigned char PositionVide;
	
	FermerBarriere();							// On ferme la barriere pour empecher les balles d'entrer
	PositionVide = SlotPretDeEntree (Slot);		// On cherche le slot vide le plus proche
	if (PositionVide==6) return PositionVide;	// Si il n'en existe pas on retourne 6 (erreur)
	PlaceBarillet (PositionVide);				// Place le slot vide devant l'entre	
	NumSlotDesire = PositionVide;				// Met la position  jour	
	return PositionVide;						// Retourne la position courante
}







// Fonction qui effectue une dpose des balles du barillet chez nous
/* 
 * On ferme la barriere quand on execute cette action
 */
void Depose_Chez_Nous(volatile unsigned char *Slot)
{
	static char Derniere_Couleur_Config=3;
	char Derniere_Couleur;
	unsigned char NbBlanche=NbBalle_Barillet(Slot,BLANC);	// Compte le nombre de balles blanches
															// Compte le nombre de balles de notre couleur
	unsigned char NbCouleur=NbBalle_Barillet(Slot,4-Couleur_Adversaire);
	char Config=Look_Up_NbBalles[NbBlanche][NbCouleur];		// Calcule la config en fonction des balles
	int i;
	

	if (Couleur_Adversaire==BLEU)							// Selon la couleur de l'adversaire
		Servo_Option_Droite();								// On place le servo  droite
	else
		Servo_Option_Gauche();								// Ou  gauche
	
	FermerBarriere();										// Ferme la barrire (si ce n'est dja fait)			
	
	for (i=0;i<5;i++)										// Parcourt les 5 balles
	{
		switch (Depose_Map[Config][Derniere_Couleur_Config][i])
		{
			case 'x' : 										// On ne dpose plus de balle
				Derniere_Couleur_Config=Derniere_Couleur;
				return; 									// On quitte la fonction
				break;
			
			case 'b' : 										// On dpose une blanche
				Depose_Blanc (Slot);						// Dpot de la blanche
				Derniere_Couleur=4;							// On mmorise l'tat de la dernire balle
				break;										
			
			case 'c' :
				if (Couleur_Adversaire==BLEU)				// Selon la couleur de l'adversaire
					Depose_Rouge (Slot);					// On dpose rouge
				else										// ... ou ...
					Depose_Bleu (Slot);						// On dpose bleu
				Derniere_Couleur=5;							// On mmorise l'tat de la dernire dpose
				break;
		}
		Delay10KTCYx(30);									// Petit dlai pour que les balles puissent rouler un peu ...

		if (Buffer_Ordre[(Indice_Courant+1)%TAILLE_BUFFER]==STOP_BARILLET)	// Si on commande un arrt du barillet, on quitte
			return;
	}
	Derniere_Couleur_Config=Derniere_Couleur;				// Mmorise la dernire balle dpose	
}	





// Fonction qui effectue une dpose des balles du barillet chez l'adversaire
/* 
 * On ferme la barriere quand on execute cette action
 */
void Depose_Chez_Adversaire(volatile unsigned char *Slot)
{
	static char Derniere_Couleur_Config=2;
	char Derniere_Couleur;
	unsigned char NbBlanche=NbBalle_Barillet(Slot,BLANC);	// Compte le nombre de balles blanches
															// Compte le nombre de balles de notre couleur
	unsigned char NbCouleur=NbBalle_Barillet(Slot,4-Couleur_Adversaire);
	char Config=Look_Up_NbBalles[NbBlanche][NbCouleur];		// Calcule la config en fonction des balles
	int i;
	

	if (Couleur_Adversaire==BLEU)							// Selon la couleur de l'adversaire
		Servo_Option_Gauche();								// On place le servo  droite
	else
		Servo_Option_Droite();								// Ou  gauche
	
	FermerBarriere();										// Ferme la barrire (si ce n'est dja fait)			
	
	for (i=0;i<5;i++)										// Parcourt les 5 balles
	{
		switch (Depose_Map[Config][Derniere_Couleur_Config][i])
		{
			case 'x' : 										// On ne dpose plus de balle
				Derniere_Couleur_Config=Derniere_Couleur;
				return; 									// On quitte la fonction
				break;
			
			case 'b' : 										// On dpose une blanche
				Depose_Blanc (Slot);						// Dpot de la blanche
				Derniere_Couleur=1;							// On mmorise l'tat de la dernire balle
				break;										
			
			case 'c' :
				if (Couleur_Adversaire==BLEU)				// Selon la couleur de l'adversaire
					Depose_Rouge (Slot);					// On dpose rouge
				else										// ... ou ...
					Depose_Bleu (Slot);						// On dpose bleu
				Derniere_Couleur=0;							// On mmorise l'tat de la dernire dpose
				break;
		}		
		Delay10KTCYx(30);									// Petit dlai pour que les balles puissent rouler un peu ...
		if (Buffer_Ordre[(Indice_Courant+1)%TAILLE_BUFFER]==STOP_BARILLET)	// Si on commande un arrt du barillet, on quitte
			return;

	}
	Derniere_Couleur_Config=Derniere_Couleur;				// Mmorise la dernire balle dpose	
}	



// --------------------------------------------------------------
// Fonction pour dposer une balle bleue
/* Dpose dans balle du ct droit 'd' ou gauche 'g', pass en paramtre
 * Cherche la balle bleue la plus proche, l'amne devant le servo de depose
 * Puis dpose la balle en la supprimant du tableau.
 * On ferme la barriere si on doit changer de slot
 * Si la fonction retourne 0, tout s'est bien pass
 * Si un problme est survenu, la fonction retourne 1 (Barillet bloqu ?)
 */
unsigned char Depose_Bleu (volatile unsigned char *Slot)
{
	unsigned char PositionBalle;
	unsigned char i=0;
	FermerBarriere();										// Ferme la barrire (si ce n'est dja fait)
	PositionBalle = SlotPretDeDepose(Slot,BLEU);			// On cherche la balle bleu la plus proche
	if (PositionBalle==6) return 0;							// Si il n'existe pas de balle bleu alors on quitte la fonction
	while (i++<3 && PlaceBarillet((PositionBalle+2)%5))		// Place la balle bleu devant le mcanisme de dpose (en cas de bloquage, on retente 3 fois)
		Delay10KTCYx(255);									// Laisse le barillet se reposer en cas de problme
	if (i>=3) return 1;										// Un problme est survenu
	Servo_Depose();											// On dpose la balle	
	Servo_Depose();											// On dpose la balle	
	Slot[(NumSlotReel+3)%5] = 0; 							// On supprime la balle du tableau
	return 0;
}


//Fonction pour dposer une balle Blanche
/* Dpose dans balle du ct droit 'd' ou gauche 'g', pass en paramtre
 * Cherche la balle blanche la plus proche, l'amne devant le servo de depose
 * Puis dpose la balle en la supprimant du tableau.
 * On ferme la barriere si on doit changer de slot
 * Si la fonction retourne 0, tout s'est bien pass
 * Si un problme est survenu, la fonction retourne 1 (Barillet bloqu ?)
 */
unsigned char Depose_Blanc (volatile unsigned char *Slot)
{
	unsigned char PositionBalle;
	unsigned char i=0;
	FermerBarriere();										// ferme la barrire si ce n'est dja fait
	PositionBalle = SlotPretDeDepose(Slot,BLANC);			// On cherche la balle bleu la plus proche
	if (PositionBalle==6) return 0;							// Si il n'existe pas de balle bleu alors on quitte la fonction
	while (i++<3 && PlaceBarillet ((PositionBalle+2)%5))	// Place la balle bleu devant le mcanisme de dpose (en cas de bloquage, on retente 3 fois)
		Delay10KTCYx(255);									// Laisse le barillet se reposer en cas de problme
	if (i>=3) return 1;										// Un problme est survenu
	Servo_Depose();											// On dpose la balle	
	Servo_Depose();											// On dpose la balle	
	Slot[(NumSlotReel+3)%5] = 0; 							// On supprime la balle du tableau
	return 0;
}

//fonction pour dposer une balle rouge
/* Dpose dans balle du ct droit 'd' ou gauche 'g', pass en paramtre
 *Cherche la balle rouge la plus proche, l'amne devant le servo de depose
 *Puis dpose la balle en la supprimant du tableau.
 *On ferme la barriere si on doit changer de slot
 * Si la fonction retourne 0, tout s'est bien pass
 * Si un problme est survenu, la fonction retourne 1 (Barillet bloqu ?)
 */
unsigned char Depose_Rouge (volatile unsigned char *Slot)
{
	unsigned char PositionBalle;
	unsigned char i=0;
	FermerBarriere();										// Ferme la barrire si ce n'est dja fait	
	PositionBalle = SlotPretDeDepose(Slot,ROUGE);			// On cherche la balle bleu la plus proche
	if (PositionBalle==6) return 0;							// Si il n'existe pas de balle bleu alors on quitte la fonction
	while (i++<3 && PlaceBarillet ((PositionBalle+2)%5))	// Place la balle bleu devant le mcanisme de dpose (en cas de bloquage, on retente 3 fois)
		Delay10KTCYx(255);									// Laisse le barillet se reposer en cas de problme
	if (i>=3) return 1;										// Un problme est survenu
	Servo_Depose();											// On dpose la balle	
	Servo_Depose();											// On dpose la balle	
	Slot[(NumSlotReel+3)%5] = 0; 							// On supprime la balle du tableau
	return 0;
}




//fonction pour tire une balle bleue
/* Cherche la balle bleue la plus proche, l'amne devant le servo de tir
 * Puis tire la balle en la supprimant du tableau.
 * On ferme la barriere si on doit changer de slot
 * Si la fonction retourne 0, tout s'est bien pass
 * Si un problme est survenu, la fonction retourne 1 (Barillet bloqu ?)
 */
unsigned char Tir_Bleu (volatile unsigned char *Slot)
{		
	unsigned char PositionBalle;
	unsigned char i=0;
	FermerBarriere();										// ferme la barrire si ce n'est dja fait
	PositionBalle = SlotPretDeTir(Slot,BLEU);				// On cherche la balle bleu la plus proche
	if (PositionBalle==6) return 0;							// Si il n'existe pas de balle bleu alors on quitte la fonction
	while (i++<3 && PlaceBarillet ((PositionBalle+3)%5))	// Place la balle bleu devant le mcanisme de tir (en cas de bloquage, on retente 3 fois)
		Delay10KTCYx(255);									// Laisse le barillet se reposer en cas de problme
	if (i>=3) return 1;										// Un problme est survenu	
//	Delay10KTCYx(250);										// On attends un peu		
	Servo_Tir();											// On dpose la balle	
	Servo_Tir();											// On dpose la balle	
	Slot[(NumSlotReel+2)%5] = 0; 							// On supprime la balle du tableau
	return 0;
}


//fonction pour tire une balle blanche
/* Cherche la balle blanche la plus proche, l'amne devant le servo de tir
 * Puis tire la balle en la supprimant du tableau.
 * On ferme la barriere si on doit changer de slot
 * Si la fonction retourne 0, tout s'est bien pass
 * Si un problme est survenu, la fonction retourne 1 (Barillet bloqu ?)
 */
unsigned char Tir_Blanc (volatile unsigned char *Slot)
{
	unsigned char PositionBalle;
	unsigned char i=0;
	FermerBarriere();										// ferme la barrire si ce n'est dja fait
	PositionBalle = SlotPretDeTir(Slot,BLANC);				// On cherche la balle bleu la plus proche
	if (PositionBalle==6) return 0;							// Si il n'existe pas de balle bleu alors on quitte la fonction
	while (i++<3 && PlaceBarillet ((PositionBalle+3)%5))	// Place la balle bleu devant le mcanisme de tir (en cas de bloquage, on retente 3 fois)
		Delay10KTCYx(255);									// Laisse le barillet se reposer en cas de problme
	if (i>=3) return 1;										// Un problme est survenu	
//	Delay10KTCYx(250);										// On attends un peu		
	Servo_Tir();											// On dpose la balle	
	Servo_Tir();											// On dpose la balle	
	Slot[(NumSlotReel+2)%5] = 0; 							// On supprime la balle du tableau
	return 0;
}

//fonction pour tire une balle rouge
/* Cherche la balle rouge la plus proche, l'amne devant le servo de tir
 * Puis tire la balle en la supprimant du tableau.
 * On ferme la barriere si on doit changer de slot
 * Si la fonction retourne 0, tout s'est bien pass
 * Si un problme est survenu, la fonction retourne 1 (Barillet bloqu ?)
 */
unsigned char Tir_Rouge (volatile unsigned char *Slot)
{		
	unsigned char PositionBalle;
	unsigned char i=0;
	FermerBarriere();										// ferme la barrire si ce n'est dja fait	
	PositionBalle = SlotPretDeTir(Slot,ROUGE);				// On cherche la balle bleu la plus proche
	if (PositionBalle==6) return 0;							// Si il n'existe pas de balle bleu alors on quitte la fonction
	while (i++<3 && PlaceBarillet ((PositionBalle+3)%5))	// Place la balle bleu devant le mcanisme de tir (en cas de bloquage, on retente 3 fois)
		Delay10KTCYx(255);									// Laisse le barillet se reposer en cas de problme
	if (i>=3) return 1;										// Un problme est survenu	
//	Delay10KTCYx(250);										// On attends un peu		
	Servo_Tir();											// On dpose la balle	
	Servo_Tir();											// On dpose la balle	
	Slot[(NumSlotReel+2)%5] = 0; 							// On supprime la balle du tableau
	return 0;
}
