// Fichier Actions_Barillet.h
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *              En-tete du module de Barillet           *
 *                                                      *
 * .Date de cration : 11 Fvrier 2008                  *
 * .Auteur : Cardineau Matthieu							*       
 *                                                      *
 * .Descriptions :                                      *
 *      -librairie pour gerer les actions de la machine *
 * a tats du barillet									*
 * 														*
 *                                                      *
 * .Grandes modifications :                             *
 *  __________________________________________________  *
 * |   DATE   |   Auteur   |          Detail          | *
 * |--------------------------------------------------| *
 * | 11/02/08 | M.Cardineau| Entete ajoute           | *
 * |          |            | -Stocker                 | *
 * |          |            | -Deposer_Bleu/blanc/rouge| *
 * |--------------------------------------------------| *
 * | 13/02/08 | M.Cardineau| -Tir_Bleu/blanc/rouge    | *
 * |--------------------------------------------------| *
 * | 18/02/08 | M.Cardineau| -SlotVideEntree		  | *
 * |--------------------------------------------------| *
 * |          |            |                          | *
 * |--------------------------------------------------| *
 * |__________|____________|__________________________| *
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *														*
 ********************************************************/

//fonction pour initialiser le barillet
/*Vide le tableau Slot
 */
unsigned char Barillet_Init(volatile unsigned char *Slot);

//fonction pour vider le barillet
/*depose pour tout les slot
 */
void Vider_Barillet(volatile unsigned char *Slot);

//Fonction pour stocker une balle que l'on detecte.
/* On passe au slot suivant et on stocke la balle dans le tableau
 * Le premier argument est le tableau Slot contenu du barillet
 * Le second argument est la couleur joue par l'averssaire
 * 
 */
void Stocker (volatile unsigned char *Slot, volatile unsigned char Couleur_Adversaire);

//fonction pour mettre un slot vide a l'entre
/*Cherche le slot vide le plus proche et positionne le barillet en fonction
 */
unsigned char SlotVideEntree (volatile unsigned char *Slot);


 
 // Fonction qui effectue une dpose des balles du barillet chez nous
/* 
 * On ferme la barriere quand on execute cette action
 */
void Depose_Chez_Nous(volatile unsigned char *Slot);


// Fonction qui effectue une dpose des balles du barillet chez l'adversaire
/* 
 * On ferme la barriere quand on execute cette action
 */
void Depose_Chez_Adversaire(volatile unsigned char *Slot);


//fonction pour dposer une balle bleue
/*Cherche la balle bleue la plus proche, l'amne devant le servo de depose
 *Puis dpose la balle en la supprimant du tableau.
 */
unsigned char Depose_Bleu (volatile unsigned char *Slot);

//fonction pour dposer une balle Blanche
/*Cherche la balle blanche la plus proche, l'amne devant le servo de depose
 *Puis dpose la balle en la supprimant du tableau.
 */
unsigned char Depose_Blanc (volatile unsigned char *Slot);

//fonction pour dposer une balle rouge
/*Cherche la balle rouge la plus proche, l'amne devant le servo de depose
 *Puis dpose la balle en la supprimant du tableau.
 */
unsigned char Depose_Rouge (volatile unsigned char *Slot);

//fonction pour tire une balle bleue
/*Cherche la balle bleue la plus proche, l'amne devant le servo de tir
 *Puis tire la balle en la supprimant du tableau.
 */
unsigned char Tir_Bleu (volatile unsigned char *Slot);

//fonction pour tire une balle blanche
/*Cherche la balle blanche la plus proche, l'amne devant le servo de tir
 *Puis tire la balle en la supprimant du tableau.
 */
unsigned char Tir_Blanc (volatile unsigned char *Slot);

//fonction pour tire une balle rouge
/*Cherche la balle rouge la plus proche, l'amne devant le servo de tir
 *Puis tire la balle en la supprimant du tableau.
 */
unsigned char Tir_Rouge (volatile unsigned char *Slot);


