// Fichier Barillet.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *              En-tete du module de Barillet           *
 *                                                      *
 * .Date de cration : 21 janvier 2008                  *
 * .Auteur : Cardineau Matthieu							*       
 *                                                      *
 * .Descriptions :                                      *
 *      -Asservissement du barillet				 		*
 *                                                      *
 * .Grandes modifications :                             *
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *                                                      *
 *                                                      *
 ********************************************************/
 
 
#include <p18f452.h>
#include <timers.h>			// Timer pour la base de temps
#include <math.h>


 
#include "barillet.h"		// Entte de la librairie
#include "Moteur.h"			// Librairie de gestion du moteur
#include "codeur.h"			// Librairie pour le codeur
#include "led.h"			// Librairie pour les leds
#include "fourche.h"		// Librairie pour les fourches



volatile char 							Flag_Jammed=0;			// Flag qui passe  un si le barillet est bloqu
volatile char 							Flag_Calle=0;			// Flag qui passe  un si le barillet est correctement call
volatile char 							State_Barillet=0;		// Etat courant du barillet
volatile int							Consigne_Position=0;	// Consigne de position pour l'asservissement


extern unsigned char 					NumSlotReel;			// Numro du slot courant

// --------------------------------------------------------------------------
/* Fonction qui initialise l'asservissement du barillet
 * Pas d'arguments
 * Initialise le timer, l'interruption, le moteur et le codeur
 * Retourne :
 *  0 : tout s'est bien pass
 *  1 : le barillet est bloqu
*/
char Init_Barillet(void)
{
	Moteur_Init();							// Initialise la commande du moteur (les E/S + le PWM)
	Codeur_Init();							// Initialise le codeur (E/S + compteurs)
			
	OpenTimer1(	TIMER_INT_ON 	&			//Autorise les interruptions. Met le bit TMR1IE de PIE1  1.
				T1_16BIT_RW 	&   		//Mode 16 bits
				T1_SOURCE_INT	&   		//Horloge interne
				T1_PS_1_1		&			//Prescaler  1
				T1_OSC1EN_OFF	&   		//Pas d'oscillateur
				T1_SYNC_EXT_OFF	); 			//Pas de synchro externe

	/***** Dfinitions des bits pour les interruptions *******/
	
	WriteTimer1(T_TIMER1);					// Charge le compteur du timer 3 avec la valeur de T_Timer3
	PIR1bits.TMR1IF = 0;					//Eteint le flag du timer 1. 0->dclenchement 
	IPR1bits.TMR1IP = 0;					//Interruption sur timer1 en priorit basse (adresse: 0x18)
	PIE1bits.TMR1IE= 1 ;					//Autorise le drapeau de dbordement du Timer0




											// Autorise les interruptions
	RCONbits.IPEN=1;						// Autorise les niveaux de priorit sur les interuptions
	INTCONbits.PEIE=1;						// Autorise les interruptions de basse priorit
	INTCONbits.GIE=1;						// Autorise les interruptions de haute priorit

											// Normalement, le barillet se positionne dans l'interruption
	while (!Flag_Calle)	 // On attends que le barillet soit en place
	{										
		if (Flag_Jammed) return 1;			// Si il y a un blocage on retourne l'erreur		
	}
	return 0;								// Pas de problme, on retourne 0
} 






// --------------------------------------------------------------------------
/* Fonction qui avance le barillet d'un slot (pour le stockage des balles)
 * Pas d'arguments 
 * Retourne :
 *  0 : tout s'est bien pass
 *  1 : le barillet est bloqu et est revenu  sa position initiale
*/
char AvanceUnSlot (void)
{
	INTCONbits.PEIE=0;							// Interdit les interruptions
	Consigne_Position=6750;						// Place la nouvelle consigne
	State_Barillet=1;							// Passe dans l'tat asservissement en position
	Flag_Calle=0;								// Baisse le flag de callage
	INTCONbits.PEIE=1;							// Rautorise les interruptions

	while (!Flag_Calle)							// Tant qu'on est pas call
	{
		if (Flag_Jammed==1)						// Si on est bloqu
		{	
			INTCONbits.PEIE=0;					// On interdit les interruptions
			Consigne_Position=0;				// On retourne  la position initiale
			State_Barillet=1;					// On reste dans l'tat 1
			Flag_Calle=0;						// On laisse le flag de callage
			INTCONbits.PEIE=1;					// On rautorise les interruptions
			while (!Flag_Calle);				// On attends d'tre call
			return 1;							// On quitte en indiquant le problme
		}
	}
	NumSlotReel=(NumSlotReel+4)%5;				// On met  jour la position du barillet
	return 0;									// Tout s'est bien pass
}	




// --------------------------------------------------------------------------
/* Fonction qui place sur le slot dsir 
 * Le numro de slot dsir devant l'entre des balles
 * Retourne :
 *  0 : tout s'est bien pass
 *  1 : le barillet est bloqu et est revenu  sa position initiale
*/
char PlaceBarillet (char NumSlotDesire)				// Place la balle bleu devant le mcanisme de dpose
{
	int Position;
	
	if (NumSlotDesire == NumSlotReel) return (0);				// Vrifie qu'on ne soit pas dj en place
	if(SensRotation(NumSlotDesire,NumSlotReel)) 				// Si la fonction retourne 1 alors le sens positif est le plur rapide
	{
		Position=6750*( (NumSlotReel-NumSlotDesire+5)%5 );		// Calcule de combien de points il faut avancer
		INTCONbits.PEIE=0;										// Interdit les interruptions
		Consigne_Position=Position;								// Place la nouvelle consigne
		State_Barillet=1;										// Passe dans l'tat asservissement en position
		Flag_Calle=0;											// Baisse le flag de callage
		INTCONbits.PEIE=1;										// Rautorise les interruptions
	}
	else 
	{
		Position=-6750*( (NumSlotDesire-NumSlotReel+5)%5 );		// Calcule de combien de points il faut avancer
		INTCONbits.PEIE=0;										// Interdit les interruptions
		Consigne_Position=Position;								// Place la nouvelle consigne
		State_Barillet=1;										// Passe dans l'tat asservissement en position
		Flag_Calle=0;											// Baisse le flag de callage
		INTCONbits.PEIE=1;										// Rautorise les interruptions
	}

	while (!Flag_Calle)							// Tant qu'on est pas call
	{
		if (Flag_Jammed) 						// Si on est bloqu
		{	
			INTCONbits.PEIE=0;					// On interdit les interruptions
			Consigne_Position=0;				// On retourne  la position initiale
			State_Barillet=1;					// On reste dans l'tat 1
			Flag_Calle=0;						// On laisse le flag de callage
			INTCONbits.PEIE=1;					// On rautorise les interruptions
			while (!Flag_Calle);				// On attends d'tre call
			return 1;							// On quitte en indiquant le problme
		}
	}
	NumSlotReel=NumSlotDesire;
	return 0;									// Tout s'est bien pass	
}	

// --------------------------------------------------------------------------
/* Fonction qui retourne le nombre de balles d'une couleur dans le barillet * 
 * Retourne :
 *  le nombre de balles de la couleur demande 
*/
unsigned char NbBalle_Barillet (volatile unsigned char *Slot,unsigned char Couleur)
{
	char i;
	char Nb=0;
	for (i=0;i<5;i++)
		if (Slot[i]==Couleur) Nb++;
	return Nb;
}	


//----------------------------------------------------------------------------
// Vecteur d'interruptions de priorit basse (utilis pour le Timer)
#pragma code lowVector=0x18							// Vecteur d'interruption de priorit basse
void atInterruptLow(void)
{
	_asm goto Interrupt_Low_Priority _endasm		// On saute  la fonction MyHighInterrupt
}
#pragma code 										// Retour  la zone de code





//----------------------------------------------------------------------------
// Routine de service de l'interruption haute priorite
#pragma interruptlow Interrupt_Low_Priority
void Interrupt_Low_Priority(void)
{
	int 				Consigne_Vitesse=0;					// Consigne  appliquer sur le moteur
	static int 			Speed;								// Vitesse mesure au codeur
	static float 		SpeedOld=0;							// Ancienne valeur de la vitesse pour le calcul du modle
	float 				Error=0;							// Erreur sur la vitesse
	static float 		Commande=0;							// Commande applique au moteur
	static float 		ErrorOld=0;							// Erreur calcule  l'chantillon prcdent
	float				SpeedEstime=0;						// Vitesse estime du barillet
	
	static int			NbError=0;							// Nombre d'erreur successives
	static long int		Position=0;							// Position du barillet
	char				Fork=0;								// Variable intermdiaire utilis pour caler le barillet avec les fourches
		

	if(PIR1bits.TMR1IF)									// Vrifie qu'il s'agit bien de l'interruption du Timer 1
	{

		Led_Verte(1);									// Pour marquer le dbut de l'interruption
		WriteTimer1(T_TIMER1);							// Recharge le compteur du Timer 1
		PIR1bits.TMR1IF=0;								// Rautotise l'interruption

		switch (State_Barillet)
		{
																		// Asservissement aux fourches			
			case 0 : 	Fork=FourcheGauche()<<2+FourcheDroite();		// Concatne la valeur des capteurs
						switch (Fork) {
							case 0: Consigne_Vitesse= 75; break;			// On avance
							case 1: Consigne_Vitesse= 75; break;			// On est pile  l'oppos : on avance
							case 8: Consigne_Vitesse=-75; break;			// On est trop loin, on recule
							case 4: Consigne_Vitesse=0;						// On est pile poil !
									Flag_Calle=1;							// On passe le flag  1
									Position=0;								// On reset la position
						}
						
						break;
						
																			// Asservissement en position au codeur
			case 1 : 	Flag_Calle=0;
						Consigne_Vitesse=0.12*(Consigne_Position-Position);
						if (Consigne_Vitesse> 1023) Consigne_Vitesse= 1023;
						if (Consigne_Vitesse<-1023) Consigne_Vitesse=-1023;						
						if (fabs (Consigne_Position-Position)<20)
							State_Barillet=0;
						break;
		}
		
		SpeedEstime=b*Commande+a*(float)Speed;			// Calcule la vitesse estime		
		Speed=Codeur_Vitesse();							// Lecture de la vitesse du moteur
		Position+=Speed;								// Met la position  jour
		
		if (fabs(SpeedEstime-Speed)>Sensibilite)		// Compare le modle  la ralit
			NbError++;									// La difrence n'est pas ngligeable, on incrmente  le nombre d'erreurs
		else
			NbError=0;									// Tout est normal			
			
		Flag_Jammed=0;
		if (NbError>NbErrorMax) 
			Flag_Jammed=1;								// On a trop d'erreur succssives : on passe le flag  1
 
		Error=(float)(Consigne_Vitesse-Speed);			// Calcul l'erreur		
		
		Commande=Commande+(Ki-Kp)*ErrorOld+Kp*Error;	// Calcul le PI
		if (Commande> 1023) Commande= 1023;				// Bride la commande
		if (Commande<-1023)	Commande=-1023;						
		ErrorOld=Error;									// Mmorise l'erreur pour l'chantillon suivant
		CommandeMoteur ((int)Commande);					// Envoie la commande sur le moteur
		
		
//		printf("%d %d %d %d\n",ConsigneVitesse,(int)Commande,Speed,(int)SpeedEstime);		
//		printf ("Pos=%d\t Cons_Pos=%d\t Sta=%d\t Er=%d\n",(int)Position,(int)Consigne_Position,(int)State_Barillet,(int)Flag_Jammed);
		Led_Verte(0);									// Pour marquer la fin de l'interruption
	}
}









//-----------------------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------------------

//fonction pour trouver la balle d'une couleur la plus proche du servo depose
/*Reois : Slot = le tableau du contenu des slot du barillet
 *	coul = la couleur de la balle que l'on cherche (1 bleu 2 blanc 3 rouge)
 *Retourne : le numro du slot ou est contenu la balle voulue la plus proche.
 *	retourne 6 si pas de balle dispo.
 */
unsigned char SlotPretDeDepose (volatile unsigned char* Slot,unsigned char coul)
{	
	unsigned char result=6;

	/*	on regarde du plus loin au plus pret si la couleur de la balle correspond a notre recherche
		au final result prend la valeur du slot le plus proche qui contient la balle recherche
		si aucun slot de correspond  la couleur voulue alors result garde la valeur 6
	*/
	if ( Slot[(NumSlotReel+3)%5] == coul ) return (NumSlotReel+3)%5;
	if ( Slot[(NumSlotReel+2)%5] == coul ) return (NumSlotReel+2)%5;
	if ( Slot[(NumSlotReel+4)%5] == coul ) return (NumSlotReel+4)%5;
	if ( Slot[(NumSlotReel+1)%5] == coul ) return (NumSlotReel+1)%5;
	if ( Slot[NumSlotReel] == coul ) return NumSlotReel;
	
	return result;
}	

//fonction pour trouver le slot vide le plus proche de l'entre du barillet
/*Reois : Slot = le tableau du contenu des slot du barillet
 *Retourne : le numro du slot vide le plus proche
 *	retourne 6 si pas de slot vide
 */

unsigned char SlotPretDeEntree (volatile unsigned char* Slot)
{	
	unsigned char result=6;

	/*	on regarde du plus loin au plus pret si la couleur de la balle correspond a notre recherche
		au final result prend la valeur du slot le plus proche qui contient la balle recherche
		si aucun slot de correspond  la couleur voulue alors result garde la valeur 6
	*/
	if ( Slot[ NumSlotReel     ] == 0 ) return NumSlotReel;
	if ( Slot[(NumSlotReel+4)%5] == 0 ) return (NumSlotReel+4)%5;
	if ( Slot[(NumSlotReel+1)%5] == 0 ) return (NumSlotReel+1)%5;
	if ( Slot[(NumSlotReel+3)%5] == 0 ) return (NumSlotReel+3)%5;
	if ( Slot[(NumSlotReel+2)%5] == 0 ) return (NumSlotReel+3)%5;
	
	return result;
}

//fonction pour trouver la balle d'une couleur la plus proche du servo de tir
/*Reois : Slot = le tableau du contenu des slot du barillet
 *	coul = la couleur de la balle que l'on cherche (1 bleu 2 blanc 3 rouge)
 *Retourne : le numro du slot ou est contenu la balle voulue la plus proche.
 *	retourne 6 si pas de balle dispo.
 */
unsigned char SlotPretDeTir (volatile unsigned char* Slot, unsigned char coul)
{	
	unsigned char result=6;

	/*	on regarde du plus loin au plus pret si la couleur de la balle correspond a notre recherche
		au final result prend la valeur du slot le plus proche qui contient la balle recherche
		si aucun slot de correspond  la couleur voulue alors result garde la valeur 6
	*/
	if ( Slot[(NumSlotReel+2)%5] == coul ) return (NumSlotReel+2)%5;
	if ( Slot[(NumSlotReel+1)%5] == coul ) return (NumSlotReel+1)%5;
	if ( Slot[(NumSlotReel+3)%5] == coul ) return (NumSlotReel+3)%5;
	if ( Slot[ NumSlotReel     ] == coul ) return NumSlotReel;
	if ( Slot[(NumSlotReel+4)%5] == coul ) return (NumSlotReel+4)%5;
	
	return result;
}




//fonction pour dterminer le sens de rotation le plus cours du barillet
/* 	Reois : 	depart : le numro du slot d'ou l'on part
				arriv : le numro du slot ou l'on va
	Retourne : sens de rotation => 1 = positif 0 = ngatif.
*/
unsigned char SensRotation (unsigned char depart, unsigned char arrive)
{
	if (((5 + arrive - depart)%5) > 2) return 0;
	//si on doit faire plus de 2 slot dans le sens trigo alors reculer est plus rapide
	else return 1; //sinon aller en marche arrire est plus court.
}

