// Fichier Barillet.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *              En-tete du module de Barillet           *
 *                                                      *
 * .Date de cration : 21 janvier 2008                  *
 * .Auteur : Cardineau Matthieu							*       
 *                                                      *
 * .Descriptions :                                      *
 *      -Asservissement du barillet				 		*
 *                                                      *
 * .Grandes modifications :                             *
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *                                                      *
 *                                                      *
 ********************************************************/
 
#ifndef __BARILLET_H_ 
#define __BARILLET_H_
 
// --------------------------------------------------------------------------
// Constantes
#define 	T_TIMER1 	15535			   	// 1 ms avant dbordement du Timer
											// Priode du quartz : 1/20e6 = 50ns
											// Priode  l'entre du prescaler : 50ns.4 = 200ns
											// Priode  la sortie du prescaler : 200ns.1 = 200ns
											// Nombre de priodes avant dbordement du timer : 10000 incrments
											// Priode entre deux interruptions : 5000*200ns = 1ms
											// Priode de 1ms -> 1KHz
											// 60536 pour 1ms
											// 55536 pour 2ms
											// 15535 pour 10ms
 
 
 #define	Kp			1.25330605			// Coefficient proportionnel
 #define 	Ki			0.432214091			// Coefficient intgral
 #define	a 			0.6551408248		// Coefficient a du modle
 #define	b 			0.27515957652		// Coefficient b du modle
 
 #define	Sensibilite	100					// Diffrence entre le modle et la vitesse relle
 #define	NbErrorMax	30					// Nombre d'erreurs avant coupure du barillet
 
 
 
 
 
// --------------------------------------------------------------------------
/* Fonction qui initialise l'asservissement du barillet
 * Pas d'arguments
 * Initialise le timer, l'interruption, le moteur et le codeur
 * Retourne :
 *  0 : tout s'est bien pass
 *  1 : le barillet est bloqu
*/
char Init_Barillet(void);




// --------------------------------------------------------------------------
/* Fonction qui avance le barillet d'un slot (pour le stockage des balles)
 * Pas d'arguments 
 * Retourne :
 *  0 : tout s'est bien pass
 *  1 : le barillet est bloqu et est revenu  sa position initiale
*/
char AvanceUnSlot (void);



// --------------------------------------------------------------------------
/* Fonction qui place sur le slot dsir 
 * Le numro de slot dsir devant l'entre des balles
 * Retourne :
 *  0 : tout s'est bien pass
 *  1 : le barillet est bloqu et est revenu  sa position initiale
*/
char PlaceBarillet (char NumSlotDesire);



// --------------------------------------------------------------------------
/* Fonction qui retourne le nombre de balles d'une couleur dans le barillet * 
 * Retourne :
 *  le nombre de balles de la couleur demande 
*/
unsigned char NbBalle_Barillet (volatile unsigned char *Slot,unsigned char Couleur);




// Prototype de la routine de service d'interuptions de priorit basse
void Interrupt_Low_Priority(void);					// Utilise pour le timer




//fonction pour trouver la balle d'une couleur la plus proche du servo depose
/*Reois : Slot = le tableau du contenu des slot du barillet
 *	coul = la couleur de la balle que l'on cherche (1 bleu 2 blanc 3 rouge)
 *Retourne : le numro du slot ou est contenu la balle voulue la plus proche.
 *	retourne 6 si pas de balle dispo.
 */
unsigned char SlotPretDeDepose (volatile unsigned char* Slot,unsigned char coul);

//fonction pour trouver le slot vide le plus proche de l'entre du barillet
/*Reois : Slot = le tableau du contenu des slot du barillet
 *Retourne : le numro du slot vide le plus proche
 *	retourne 6 si pas de slot vide
 */
unsigned char SlotPretDeEntree (volatile unsigned char* Slot);

//fonction pour trouver la balle d'une couleur la plus proche du servo de tir
/*Reois : Slot = le tableau du contenu des slot du barillet
 *	coul = la couleur de la balle que l'on cherche (1 bleu 2 blanc 3 rouge)
 *Retourne : le numro du slot ou est contenu la balle voulue la plus proche.
 *	retourne 6 si pas de balle dispo.
 */
unsigned char SlotPretDeTir (volatile unsigned char* Slot, unsigned char coul);

//fonction pour dterminer le sens de rotation le plus cours du barillet
/* 	Reois : 	depart : le numro du slot d'ou l'on part
 *				arriv : le numro du slot ou l'on va
 *	Retourne : sens de rotation => 1 = positif 0 = ngatif.
 */
unsigned char SensRotation (unsigned char depart, unsigned char arrive);




#endif
