// Fichier Capt_Coul.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *              En-tete du module de Barillet           *
 *                                                      *
 * .Date de cration : 11 Fvrier 2008                  *
 * .Auteur : Cardineau Matthieu							*       
 *                                                      *
 * .Descriptions :                                      *
 *      -fonction pour gerer le capteur de couleur      *
 * 														*
 *                                                      *
 * .Grandes modifications :                             *
 *  __________________________________________________  *
 * |   DATE   |   Auteur   |          Detail          | *
 * |--------------------------------------------------| *
 * | 11/02/08 | M.Cardineau| Entete ajoute           | *
 * |--------------------------------------------------| *
 * |          |            |                          | *
 * |--------------------------------------------------| *
 * |          |            |                          | *
 * |--------------------------------------------------| *
 * |__________|____________|__________________________| *
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *                                                      *
 *                                                      *
 ********************************************************/

		

#define 	TEMPS_RESTANT	5000		//5000 temps de cycle max d'attente de la rponse du capteur (5000cycle = 1ms)
//------------------------------------------------------------------------------------//

//------------------------------INCLUSIONS-------------------------------//
#include <p18f452.h>	// Librairie pour le PIC
//#include <timers.h>		//pour configurer les timers
#include <delays.h>		//pour les tempos
#include <adc.h>

#include "Capt_Coul.h"
//-----------------------------------------------------------------------//

//fonction pour initialiser les port pour le capteur de couleur
/* Le bit RA0 est initialiser en sortie pour commander (mais sert d'entre pour reevoir la confirmation
 * Le bit RA1 et RA2 servent a coder la couleur sur 2 bit :
 * 0 = pas de balle dtecte 1= balle bleu 2 = blanche 3 = rouge
 */
void CaptCoul_Init (void)
{
	OpenADC (ADC_0ANA_0REF,ADC_INT_OFF);
	TRIS_CAPT_COUL_START = 0;	//initialise en sortie pour la demande
	TRIS_CAPT_COUL_D0 = 1;		//bit de donne en entre (poid faible)
	TRIS_CAPT_COUL_D1 = 1;		//bit de donne en entre (poid fort)
	CAPT_COUL_START=0;
}


//fonction pour demander et lire la couleur du slot courrant+1
/*Retourne une valeur de balle de 1 a 3
 * 1) demande l'envoie de la donne (mise a 1 pendant 5us)
 * 2) met le port en entre, lance le timer et attend la reception
 * 3) si on depasse le temps demand on quitte, sinon on lit la donne.
 */
unsigned char LireCaptCoul (void)
{	unsigned char Couleur;					// Contiendra la donne envoye par le capteur
	unsigned char i = 0;					// Variable pour le timeout.
	CAPT_COUL_START = 1;					// Active la demande
	while (!CAPT_COUL_D0 && !CAPT_COUL_D1)	// On attend que le capteur de couleur confirme l'envoie de la donne.
	{	
		i++;				//on incrmente l'attente
		if (i>250) return 0;//si on a attendu 250 fois 4us (donc 1ms) on retourne 0 (erreur)
		Delay100TCYx (10);	//on attend*/
	}
	Delay10TCYx (2);						//on attend 400us
	Couleur = CAPT_COUL_D1*2+CAPT_COUL_D0;	//on concatne les bits de donne pour former la couleur de balle.
	CAPT_COUL_START = 0;					//desactive la demande

	return Couleur;							// Retourne la couleur lue
}

