// Fichier Codeur.c
/////////////////////////////////////////////////////////

/****************************************************************
 *                                                    			*
 * .Date de cration : 11/01/08                         		*
 * .Auteur : Matthieu Cadic                             		*       
 *           Yohan Luzeau                                  		*
 * .Descriptions :                                      		*
 *      - Fichier CarteMoteur pour la gestion 			    	*
 *        du rouleau droit   									*
 *                                           					*
 * .Grandes modifications :                             		*
 *  __________________________________________________  		*
 *                                                      		*
 *                                                      		*
 * .Remarques :                                         		*
 *      - possibilit de grer un asservissement avec les 	 	*
 *		  fonctions	InterruptVectorLow et RoutineInterrupt  	*
 *      -                                    					*
 *                                                      		*
 ****************************************************************/

#include <p18f452.h>		// Pour les registres du PIC18F252
#include <timers.h>			// Timer 0 utilis en compteur 16 bits

#include "Codeur.h"		// Entte de la librairie




// --------------------------------------------------------------------------
/* Initialise le codeur
 * Configure les ports en entre
 * Configure le timer 0 en compteur
 * Initialise le compteur
*/
void Codeur_Init(void)
{
	// O -> output
	// 1 -> Input 

	TRIS_CODEUR_CANAL_A=1;			// Configure le canal A en entre
	TRIS_CODEUR_SENS=1;			// Configure le sens de rotation en entre
	

	// Configure le Timer 0 en mode compteur
	OpenTimer0 ( TIMER_INT_OFF &     						 // Interdire les interruptions
				 T0_16BIT &		    						 // Compter sur 16 bits
			   	 T0_SOURCE_EXT &    						 // Incrmentation sur source externe
				 T0_PS_1_1 );       						 // Meilleur prcision sans division de frquence

	// Initialise le compteur  0				 
	WriteTimer0 (0);               						     // Initialisation du timer  0
}



// --------------------------------------------------------------------------
/* Retourne la vitesse lue au codeur
 * La fonction lit le compteur et soustrait l'ancienne valeur
 * en tenant compte des dbordements et du signe
 * La fonction doit tre appele  intervalles rgulier pour
 * retourner une valeur fiable
*/

int Codeur_Vitesse(void)
{
	static int Compteur_Old=0;							// Variable qui sert  la mmorisation de l'ancienne valeur
	int Vitesse,Compteur;								// Vitesse et valeur courant du compteur	
	
	Compteur=ReadTimer0();								// Lit la valeur courante
	
	if( Compteur>=Compteur_Old )							// Le compteur a-t-il dbord ?
		Vitesse = Compteur - Compteur_Old;				// Non, on fait la diffrence de na nouvelle valeur moins l'ancienne
	else												// sinon (dbordement)
		Vitesse = Compteur - Compteur_Old + 0xFFFF;		// on ajoute 65535 pour compencer le dbordement
	
	Compteur_Old = Compteur;							// Mmorise l'ancienne valeur
	
	if (CODEUR_SENS)
		return (Vitesse);
	else
		return (-Vitesse);
}
