// Fichier CarteMoteur.c
/////////////////////////////////////////////////////////

/****************************************************************
 *                                                    			*
 * .Date de cration : 11/01/08                         		*
 * .Auteur : Matthieu Cadic                             		*       
 *           Yohan Luzeau                                  		*
 * .Descriptions :                                      		*
 *      - Fichier CarteMoteur pour la gestion 			    	*
 *        du rouleau droit   									*
 *                                           					*
 *                                                      		*
 *                                                      		*
 * .Remarques :                                         		*
 *      - possibilit de grer un asservissement avec les 	 	*
 *		  fonctions	InterruptVectorLow et RoutineInterrupt  	*
 *      -                                    					*
 *                                                      		*
 ****************************************************************/



#include <p18f452.h>	// Pour les registres du PIC18F252
#include <timers.h>		// Timer 2 utilis pour le PWM
#include <pwm.h>		// Pour le PWM


#include "moteur.h"		// Entte avec les constantes




// --------------------------------------------------------------------------
/* Initialise la commande des moteurs
 * Configure les ports en sortie
 * Utilise le timer 2 pour gnrer le PWM
 * Arrte les moteur
*/
void Moteur_Init(void)
{
	TRIS_MOTEUR_PWM=0;										// PWM en sortie
	TRIS_MOTEUR_DIR=0;										// Commande du sens de rotation en sortie
	
	// --- Configure le Timer 2 pour le PWM ---

	OpenTimer2 (TIMER_INT_OFF &	T2_PS_1_1 &	T2_POST_1_1); 	// Pour le PWM
	OpenPWM1(0xFF);											// Frquence du PWM : 19531 Hz 
	SetDCPWM1(0x00);										// Arrete le moteur
}




// --------------------------------------------------------------------------
/* Commande le moteur de la carte
 * Le paramtre reu est la vitesse dsire entre -1023 et +1023
 *  -1023 	: a fond en arrire
 *  0 		: moteur  l'arrt
 *  +1023	: a fond en avant 
 * Si la vitesse n'est pas comprise dans cet intervalle, elle est tronque
 * La vitesse rellement appliqu est retourne par la fonction
*/
int CommandeMoteur(int Vit)								
{
	if (Vit >= 1023)			// Si la consigne donne dans le sens positif est trop grande 
	{							
		Vit = 1023;				// On la tronque
	}
	
	if ( Vit <= -1023)			// Si la consigne donne dans le sens ngatif est trop grande
	{								
		Vit = -1023;			// On la tronque
	}

	if(Vit>=0)					// Selon le sens de rotation
	{
		SetDCPWM1(Vit);			// Rgle le rapport cyclique du PWM -> Vitesse du moteur
		MOTEUR_DIR=0;			// sens positif
	}
	else
	{
		SetDCPWM1(-Vit);		// Rgle le rapport cyclique du PWM -> Vitesse du moteur
		MOTEUR_DIR=1;			// sens ngatif
	}	
	return Vit;				// On retourne la vitesse tronque
}
