// Fichier Servos.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *              En-tete du module de servo              *
 *                                                      *
 * .Date de cration : 19/12/07                         *
 * .Auteur : Jrmy Gardais                             *       
 *                                                      *
 * .Descriptions :                                      *
 *      - code des fonctions pour les servos            *
 *                                                      *
 * .Grandes modifications :                             *
 *  __________________________________________________  *
 * |   DATE   |   Auteur   |          Detail          | *
 * |--------------------------------------------------| *
 * | 19/12/07 | J.Gardais  | Cration et code de la   | *
 * |          |&P.Lucidarme| fonctions: Servo_Init    | *
 * |          |            | InterruptVectorLow       | * 
 * |          |            | ServiceInterruptLow      | *
 * |          |            | StateMachineServo        | *
 * |--------------------------------------------------| *
 * | 23/01/08 |M.Cardineau | -ajout de la config du   | *
 * |          |            |timer1 et des ports       | *
 * | 24/01/08 |            | -Servo_Depose()          | *
 * |          |            | -Servo_Tir()             | * 
 * |          |            | -Interuption_Servomoteur() *
 * |--------------------------------------------------| *
 * | 18/02/08 |M.Cardineau | -OuvrirBarriere()        | *
 * |          |            | -FermerBarriere()        | *
 * |--------------------------------------------------| *
 * |          |            |                          | *
 * |--------------------------------------------------| *
 * |__________|____________|__________________________| *
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *      - Commenter!                                    *
 *      - La librairie utilise le timer1 pour commander *
 * les servomoteurs                                     *
 *      - 0 dans la commande d'un servomoteur			*
 *                                                      *
 ********************************************************/

//------INCLUSIONS-------//
#include <p18f452.h>		//librairie du PIC
#include <timers.h>			//librairies pour les timer du PIC
#include <delays.h>

#include "servos.h"


// --------------------------------------------------------------------------
/* Fonction qui initialise les servomoteurs en position initialie
 * La fonction retourne 0 si tout se passe bien
 * En cas de problme, la fonction retourne 1
*/
unsigned char Servo_Init(void)
{
	int i=0;
	
	TRIS_SERVO_DEPOSE=0;							// Configure tous les ports en entre
	TRIS_SERVO_CANON=0;
	TRIS_SERVO_BARRIERE=0;
	TRIS_SERVO_OPTION=0;
	
	SERVO_DEPOSE=0;									// Place les commandes des servos au repos
	SERVO_CANON=0;
	SERVO_BARRIERE=0;
	SERVO_OPTION=0;

	return (0);
}	

// Place le servo pour diriger les balles  droite
void Servo_Option_Droite()
{
	SERVO_OPTION=0;
}	

// Place le servo pour diriger les balles  gauche
void Servo_Option_Gauche()
{
	SERVO_OPTION=1;
}	


//permet de dposer une balle (sortie puis rentre du doigt)
void Servo_Depose(void)
{
	SERVO_DEPOSE=1;
	Delay10KTCYx(70);
	SERVO_DEPOSE=0;
	Delay10KTCYx(70);
}



//permet de tirer une balle (sortie puis rentre du doigt)
void Servo_Tir(void)
{
	SERVO_CANON=1;
	Delay10KTCYx(100);
	SERVO_CANON=0;
	Delay10KTCYx(100);
}

//met le servo barriere en position basse (les balles peuvent passer)
void OuvrirBarriere(void)
{
	SERVO_BARRIERE=0;
}

//met le servo barriere en position haute (les balles sont bloques)
void FermerBarriere(void)
{
	SERVO_BARRIERE=1;
}




