// Fichier Servos.h
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *              En-tete du module de servo              *
 *                                                      *
 * .Date de cration : 19/12/07                         *
 * .Auteur : Jrmy Gardais                             *       
 *                                                      *
 * .Descriptions :                                      *
 *      - Librairie qui gre les 4 servos moteurs de    *
 * la carte barillet                                    *
 *                                                      *
 * .Grandes modifications :                             *
 *  __________________________________________________  *
 * |   DATE   |   Auteur   |          Detail          | *
 * |--------------------------------------------------| *
 * | 19/12/07 | J.Gardais  | Cration et code de la   | *
 * |          |&P.Lucidarme| fonctions: Servo_Init    | *
 * |          |            | InterruptVectorLow       | * 
 * |          |            | ServiceInterruptLow      | *
 * |          |            | StateMachineServo        | *
 * |--------------------------------------------------| *
 * | 23/01/08 |M.Cardineau | -ajout de la config du   | *
 * |          |            |timer1 et des ports       | *
 * | 24/01/08 |            | -Servo_Depose()          | *
 * |          |            | -Servo_Tir()             | * 
 * |          |            | -Interuption_Servomoteur() *
 * |--------------------------------------------------| *
 * | 18/02/08 |M.Cardineau | -OuvrirBarriere()        | *
 * |          |            | -FermerBarriere()        | *
 * |--------------------------------------------------| *
 * |          |            |                          | *
 * |--------------------------------------------------| *
 * |__________|____________|__________________________| *
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *      - Commenter!                                    *
 *      - La librairie utilise le timer1 pour commander *
 * les servomoteurs                                     *
 *      - 0 dans la commande d'un servomoteur			*
 *                                                      *
 ********************************************************/

#ifndef __servo__			// pour viter plusieur appels   la fonction
#define __servo__

//--------------------------DEFINITIONS-----------------------------------------------//

#define SERVO_DEPOSE 		PORTDbits.RD3			//Port de sortie du servo ejecteur	
#define SERVO_CANON 		PORTDbits.RD2			//Port de sortie du servo de tir
#define SERVO_BARRIERE 		PORTDbits.RD0			//Port de sortie du servo droit
#define SERVO_OPTION 		PORTDbits.RD1			//Port de sortie du servo gauche

#define TRIS_SERVO_DEPOSE	TRISDbits.TRISD3		//Pour configurer les Servomoteurs
#define TRIS_SERVO_CANON 	TRISDbits.TRISD2
#define TRIS_SERVO_BARRIERE	TRISDbits.TRISD0
#define TRIS_SERVO_OPTION 	TRISDbits.TRISD1


// Place le servo pour diriger les balles  droite
void Servo_Option_Droite(void);



// Place le servo pour diriger les balles  gauche
void Servo_Option_Gauche(void);



// --------------------------------------------------------------------------
/* Fonction qui initialise les servomoteurs en position initialie
 * La fonction retourne 0 si tout se passe bien
 * En cas de problme, la fonction retourne 1
*/
unsigned char Servo_Init(void);




//permet de dposer une balle (sortie puis rentre du doigt)
void Servo_Depose(void);



//permet de tirer une balle (sortie puis rentre du doigt)
void Servo_Tir(void);



//met le servo barriere en position basse (les balles peuvent passer)
void OuvrirBarriere(void);



//met le servo barriere en position haute (les balles sont bloques)
void FermerBarriere(void);




#endif __servo__
