// Fichier I2C_Slave.h
/////////////////////////////////////////////////////////

/****************************************************************
 *                                                    			*
 * .Date de cration : 11/01/08                         		*
 * .Auteur : Guibert David                              		*       
 *           Cadic Matthieu										*
 *           Luzeau Yohan                                		*
 * .Descriptions :                                      		*
 *      - Librairie pour la gestion de l'I2C en mode esclave   	*
 *                                                      		*
 *                                                      		*
 *                                                      		*
 * .Remarques :                                         		*
 *      - Gestion des ordre reus par interruption plutt		*
 *		  que par polling								   		*
 *      - Adresse de la carte droite 0x4E          				*
 *                                                      		*
 ****************************************************************/
 
 

#define 	SLAVE_ADRESS	0x86					// Adresse I2C de la carte


#define 	TRIS_SCL 		TRISCbits.TRISC3		// Registre de direction du port SCL
#define 	TRIS_SDA 		TRISCbits.TRISC4		// Registre de direction du port SDA



//------------------------------DEFINITIONS-------------------------------//

#define JOUER_BLEU	 			0xE0					//ordre de changer de couleur pour jouer bleu
#define VIDER					0xD2					//ordre de vider le barillet et de jouer rouge

#define	DEPOSER_CHEZ_NOUS		0x40					// Ordre de faire une dpose chez nous
#define	DEPOSER_CHEZ_ADV		0x2C					// Ordre de faire une dpose chez l'adversaire


#define DEPOSER_BLEU			0x56					//ordre de dposer une balle bleue vers la droite du robot
#define DEPOSER_BLANC			0xB8					//ordre de dposer une balle blanche vers la droite du robot
#define DEPOSER_ROUGE			0xA6					//ordre de dposer une balle rouge vers la droite du robot

#define SERVO_GAUCHE			0x47					//ordre de dposer une balle bleue vers la gauche du robot
#define SERVO_DROITE			0x23					//ordre de dposer une balle blanche vers la gauche du robot
		
#define TIR_BLEU 				0x7E					//ordre de tirer une balle bleue
#define TIR_BLANC 				0x98					//ordre de tirer une balle blanche
#define TIR_ROUGE				0x46					//ordre de tirer une balle rouge

#define	STOP_BARILLET			0x32					// Ordre d'arrt du barillet
#define TAILLE_BUFFER 			10						//taille du tableau contenant les ordres

//------------------------------------------------------------------------//



// -----------------------------------------------------------------------
/* Initialise le module I2C
 * Configure les ports entre
 * Configure le module en esclave
 * Spcifie l'adresse et configure les interruptions
*/
void I2C_Init(void);



//----------------------------------------------------------------------------
// Routine de service de l'interruption haute priorite
void ServiceInterruptHigh(void);



// -----------------------------------------------------------------------
/* Fonction appelle lors d'un interruption sur l'I2C
 * Le paramtre pass correspond  la valeur  envoyer en cas de transmission
 * La fonction retourne la dernire valeur lue lors de la dernire reception
*/
char InterruptI2C(void);
