// Fichier main.c
/////////////////////////////////////////////////////////

/****************************************************************
 *                                                    			*
 * .Date de cration : 11/01/08                         		*
 * .Auteur : Guibert David                              		*
 *           Cadic Matthieu										*
 *           Luzeau Yohan                                		*
 *			 Philippe Lucidarme									*
 *			 Matthieu Cardineau								    *
 * .Descriptions :                                      		*
 *      - Fichier principal pour la gestion du barillet		   	*
 *                                                      		*
 *                                                      		*
 *                                                      		*
 * .Remarques :                                         		*
 *																*
 ****************************************************************/




// ------------ Directives de compilation
#pragma config WDT = OFF	// Pas de  watchdog
#pragma config OSC = HS		// Quartz HS pour un quartz 20MHz
#pragma config LVP = OFF	// Low voltage programming disabled



// --------------------------------------------------------------------------
// Enttes ncessaires
#include <p18f452.h>			// Pour les registres du PIC18F252
#include <delays.h>    			// Pour les temporisations
#include <timers.h>				// Pour la base de temps de 10ms
#include <adc.h>				// Bibliothque des convertisseurs Analogiques Numriques
#include <stdio.h>

#include "Moteur.h"				// Librairie de gestion du moteur
#include "I2C_Slave.h"			// Librairie pour l'I2C
#include "led.h"				// Librairie pour les leds
#include "codeur.h"				// Librairie pour le codeur
#include "barillet.h"			// Pour l'asservissement du barillet
#include "servos.h"				// Pour les servomoteurs
#include "BarriereIR.h"			// Pour la barrire IR de dtection des balles
#include "Capt_Coul.h"			// Pour le capteur de couleur
#include "Fourche.h"			// Pour les fourches du barillet
#include "Servos.h"				// pour la commande des servomoteurs
#include "Actions_Barillet.h"	// Action de base pour le barillet

#include "serial.h"			// Pour la liaison srie (surtout pour le dbuggage)
#include "stdio.h"




																		



// --------------------------------------------------------------------------
// Variables partage

volatile unsigned char 			NumSlotReel=0;						// Numro du slot courant
volatile unsigned char 			NumSlotDesire=0;					// Numro du slot demand

volatile unsigned char 			Slot[5]={0};						// Contenu du barillet
volatile unsigned char 			FlagStocker=0;						// 1 = en train de stocker
volatile unsigned char 			FlagBusy=0;							// 1 = en train d'executer un ordre

volatile unsigned char 			Couleur_Adversaire=1;				// Couleur joue par l'adversaire
																	// Utilise pour jecter les balles
																	// Bleu par dfaut

extern unsigned char 			Buffer_Ordre[TAILLE_BUFFER];		// Tableau qui bufferise les ordres  executer

unsigned char 					Indice_Courant=0;			// Pour pointer l'ordre courant dans le tableau des ordres


// --------------------------------------------------------------------------

// Prototype de la routine de service d'interuptions de priorit haute
void Interrupt_High_Priority(void);					// Utilise pour l'IC


// Fonction qui bloque le programme en cas de problme
// La led rouge clignote
void Error(void);






/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//                            Fonction Principale                          //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

void main (void)
{
	int i=0;								// indice utilis pour les boucles
	unsigned char State_Main=0;				// Pour l'tat de la machine  tat du main
//	unsigned char Indice_Courant=0;			// Pour pointer l'ordre courant dans le tableau des ordres
	
//	unsigned char Couleur=1;				// Couleur de l'adversaire (couleur des balles  jecter)
											// 1 = bleu
											// 3 = rouge


	// Configure le micro	

	OpenADC (ADC_0ANA_0REF,ADC_INT_OFF);	// Desactive les convertisseurs Analogiques/Numriques
	Led_Init();								// Configure les ports pour les leds
	Led_Rouge(1);							// Allume la led pour marquer le dbut de l'init
 	I2C_Init ();							// Initialise l'I2C 	
 	Fourche_Init();							// Configure les ports pour les fourches
 	CaptCoul_Init();						// Configure le capteur de couleur
 	BarriereIR_Init();						// Initialise la barrire IR de dtection des balles
 	Serial_Init();							// Initialise la liaison srie pour le dbuggage 	
 	if (Servo_Init()) Error();				// Initialise les servomoteurs 	
 	if (Init_Barillet()) Error();			// Initialise l'asservissement du barillet (En cas de problme : stoppe le programme)

 	Led_Rouge(0);							// Eteinds la led pour marquer la fin de l'init !
 	
	
	
	
	
	while(1)								// Rien a faire, tout est gr dans la routine d'interruption
	{

		//AFFICHAGE SUR TERMINAL POUR DEBUGGER-------//
		printf ("Slot={%d,%d,%d,%d,%d}\tBuffer={",Slot[0],Slot[1],Slot[2],Slot[3],Slot[4]);
		for (i=0;i<TAILLE_BUFFER;i++)
			printf ("%d %x,",i,Buffer_Ordre[i]);
		printf ("}\tNumSlotReel=%d BO=%d\n",NumSlotReel,Buffer_Ordre[Indice_Courant]);
		

		//-------------------------------------------//
		


		Led_Ambre(0);
		switch (Buffer_Ordre[Indice_Courant])	//en fonction de l'ordre courant a executer.
		{
										
			case 0 : 											// Si on a pas d'ordre a executer
				FlagBusy=0;										// On signale qu'on est libre
				
				Led_Ambre(1);
				
				if (Slot[NumSlotReel]!=0)						// Si le slot en bas n'est pas vide 							
					SlotVideEntree(Slot);						//	Alors on en met un
				else
					OuvrirBarriere();							// Quand on a un slot vide en bas, on ouvre.
	
				
				if (BarriereIR() && !Slot[NumSlotReel]) 		// Si une balle se prsente dans un slot vide
				{
					FlagBusy= 0x10;								// Pour signaler qu'on est occuper
					FlagStocker = 0x20;							// On signale que l'on est en train de stocker
					Stocker(Slot,Couleur_Adversaire);			// Stocke la balle, analyse la couleur et regarde si on conserve
					FlagStocker = 0;							// On signale qu'on a fini de stocker							
				}						
			break;
			
			
			
			case JOUER_BLEU :									// On informe la carte que l'on joue la couleur bleu
			Couleur_Adversaire=3;								// On joue la couleur bleue et l'adversaire rouge
			Buffer_Ordre[Indice_Courant]=0;						// Efface l'ordre
			Indice_Courant=(Indice_Courant+1)%TAILLE_BUFFER;	// Passe  l'ordre suivant			
			break;
			
			
				
			case VIDER :										// On demande une vidange du barillet
			FlagBusy= 0x10;										// Pour signaler qu'on est occuper
			Vider_Barillet(Slot);								// Vidange le barillet
			Buffer_Ordre[Indice_Courant]=0;						// Efface l'ordre
			Indice_Courant=(Indice_Courant+1)%TAILLE_BUFFER;	// Passe  l'ordre suivant
			Couleur_Adversaire=1;								// On joue la couleur rouge et l'adversaire joue bleu
	 		break;
	 		
	 		
	 		case DEPOSER_CHEZ_NOUS :							// On dpose chez nous
			FlagBusy= 0x10;										// Pour signaler qu'on est occuper
			Depose_Chez_Nous(Slot);									// Ordre de dposer chez nous
			Buffer_Ordre[Indice_Courant]=0;						// Efface l'ordre
			Indice_Courant=(Indice_Courant+1)%TAILLE_BUFFER;	// Passe  l'ordre suivant
	 		break;
	 		

	 		case DEPOSER_CHEZ_ADV :								// On dpose chez l'adversaire
			FlagBusy= 0x10;										// Pour signaler qu'on est occuper
			Depose_Chez_Adversaire(Slot);						// Ordre de dposer chez l'adversaire
			Buffer_Ordre[Indice_Courant]=0;						// Efface l'ordre
			Indice_Courant=(Indice_Courant+1)%TAILLE_BUFFER;	// Passe  l'ordre suivant
	 		break;

	 		
			case DEPOSER_BLEU :									// On demande de dposer une balle bleue
			FlagBusy= 0x10;										// Pour signaler qu'on est occuper
			Depose_Bleu(Slot);									// Dpose la balle bleue
			Buffer_Ordre[Indice_Courant]=0;						// Efface l'ordre
			Indice_Courant=(Indice_Courant+1)%TAILLE_BUFFER;	// Passe  l'ordre suivant
		 	break;


			case DEPOSER_BLANC :								// On demande de dposer une balle bleue
			FlagBusy= 0x10;										// Pour signaler qu'on est occuper
			Depose_Blanc(Slot);									// Dpose la balle bleue
			Buffer_Ordre[Indice_Courant]=0;						// Efface l'ordre
			Indice_Courant=(Indice_Courant+1)%TAILLE_BUFFER;	// Passe  l'ordre suivant
			break;

 	
		 	
		 	case DEPOSER_ROUGE :								// On demande de dposer une balle bleue
			FlagBusy= 0x10;										// Pour signaler qu'on est occuper
			Depose_Rouge(Slot);									// Dpose la balle bleue
			Buffer_Ordre[Indice_Courant]=0;						// Efface l'ordre
			Indice_Courant=(Indice_Courant+1)%TAILLE_BUFFER;	// Passe  l'ordre suivant
			break;





			case SERVO_GAUCHE :									// On demande de placer le servo  gauche
			FlagBusy= 0x10;										// Pour signaler qu'on est occuper
			Buffer_Ordre[Indice_Courant]=0;						// Efface l'ordre
			Indice_Courant=(Indice_Courant+1)%TAILLE_BUFFER;	// Passe  l'ordre suivant
			Servo_Option_Gauche();								// Place le servo  gauche
		 	break;

			case SERVO_DROITE :									// On demande de placer le servo  gauche
			FlagBusy= 0x10;										// Pour signaler qu'on est occuper
			Buffer_Ordre[Indice_Courant]=0;						// Efface l'ordre
			Indice_Courant=(Indice_Courant+1)%TAILLE_BUFFER;	// Passe  l'ordre suivant
			Servo_Option_Droite();								// Place le servo  gauche
		 	break;



	 		
	 		case TIR_BLEU :										// On demande de dposer une balle bleue
			FlagBusy= 0x10;										// Pour signaler qu'on est occuper
			Tir_Bleu(Slot);										// Dpose la balle bleue
			Buffer_Ordre[Indice_Courant]=0;						// Efface l'ordre
			Indice_Courant=(Indice_Courant+1)%TAILLE_BUFFER;	// Passe  l'ordre suivant
		 	break;


			case TIR_BLANC :									// On demande de dposer une balle bleue
			FlagBusy= 0x10;										// Pour signaler qu'on est occuper
			Tir_Blanc(Slot);									// Dpose la balle bleue
			Buffer_Ordre[Indice_Courant]=0;						// Efface l'ordre
			Indice_Courant=(Indice_Courant+1)%TAILLE_BUFFER;	// Passe  l'ordre suivant
		 	break;
		 	
		 	
		 	case TIR_ROUGE :									// On demande de dposer une balle bleue
			FlagBusy= 0x10;										// Pour signaler qu'on est occuper
			Tir_Rouge(Slot);									// Dpose la balle bleue
			Buffer_Ordre[Indice_Courant]=0;						// Efface l'ordre
			Indice_Courant=(Indice_Courant+1)%TAILLE_BUFFER;	// Passe  l'ordre suivant
		 	break;

	 		
	 		default : 											// Une commande inconnu ? On l'ignore !
	 		Buffer_Ordre[Indice_Courant]=0;						// Efface l'ordre
			Indice_Courant=(Indice_Courant+1)%TAILLE_BUFFER;	// Passe  l'ordre suivant					
		}		

		
	}
}








// Fonction qui bloque le programme en cas de problme
// La led rouge clignote
void Error(void)
{
	INTCONbits.PEIE=0;					// Interdit les interruptions de basse priorit
	INTCONbits.GIE=0;					// Interdit les interruptions de haute priorit
	CommandeMoteur(0);					// Arrte le moteur
	while (1)
	{
		LED_ROUGE=!LED_ROUGE;			// Fait clignoter la led rouge
		Delay10KTCYx(30);				
	}
}



