// Fichier Servos.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *              En-tete du module de servo              *
 *                                                      *
 * .Date de cration : 19/12/07                         *
 * .Auteur : Jrmy Gardais                             *       
 *                                                      *
 * .Descriptions :                                      *
 *      - code des fonctions pour les servos            *
 *                                                      *
 * .Grandes modifications :                             *
 *  __________________________________________________  *
 * |   DATE   |   Auteur   |          Detail          | *
 * |--------------------------------------------------| *
 * | 19/12/07 | J.Gardais  | Cration et code de la   | *
 * |          |&P.Lucidarme| fonctions: Servo_Init    | *
 * |          |            | InterruptVectorLow       | * 
 * |          |            | ServiceInterruptLow      | *
 * |          |            | StateMachineServo        | *
 * |--------------------------------------------------| *
 * | 23/01/08 |M.Cardineau | -ajout de la config du   | *
 * |          |            |timer1 et des ports       | *
 * | 24/01/08 |            | -Servo_Depose()          | *
 * |          |            | -Servo_Tir()             | * 
 * |          |            | -Interuption_Servomoteur() *
 * |--------------------------------------------------| *
 * | 18/02/08 |M.Cardineau | -OuvrirBarriere()        | *
 * |          |            | -FermerBarriere()        | *
 * |--------------------------------------------------| *
 * |          |            |                          | *
 * |--------------------------------------------------| *
 * |__________|____________|__________________________| *
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *      - Commenter!                                    *
 *      - La librairie utilise le timer1 pour commander *
 * les servomoteurs                                     *
 *      - 0 dans la commande d'un servomoteur			*
 *                                                      *
 ********************************************************/

//------INCLUSIONS-------//
#include <p18f252.h>		//librairie du PIC
#include <timers.h>			//librairies pour les timer du PIC
#include <delays.h>

#include "Servos.h"
//-----------------------//






/***********************************Initialisation des servo***************************************/

void Servo_Init(void)
{
	/***** Dfinition des ports en sortie *****/
	TRIS_SERVO_EJECT=0;		
	TRIS_SERVO_CANON=0; 	
	TRIS_SERVO_BARRIERE=0; 	//on configure les ports de commande des servomoteurs en sortie 	
	TRIS_SERVO_OPTION=0; 	

	OpenTimer1(	TIMER_INT_ON 	&	//Autorise les interruptions. Met le bit TMR1IE de PIE1  1.
				T1_16BIT_RW 	&   //Mode 16 bits
				T1_SOURCE_INT	&   //Horloge interne
				T1_PS_1_1		&	//Prescaler  1
				T1_OSC1EN_OFF	&   //Pas d'oscillateur
				T1_SYNC_EXT_OFF	); 	//Pas de synchro externe

	/***** Dfinitions des bits pour les interruptions *******/
	
	PIR1bits.TMR1IF = 0;	// Eteint le flag du timer 1. 0->dclenchement et interruption
}






/******Variable globales pour les pwm*******/  //A VERIFIER QUE ACCESSIBLE UNIQUEMENT AU .C
int PWM_Servo_Eject=0;
int PWM_Servo_Canon=0;	
int PWM_Servo_Barriere=0;
int PWM_Servo_Option=0;
/*********************************************/
/*
//permet de dposer une balle (sortie puis rentre du doigt)
void Servo_Depose(void)
{
	Servo_Eject_Commande(RC_DEPOSE_ACTIF);
	Delay10KTCYx(75);
	Servo_Eject_Commande(RC_DEPOSE_REPOS);
}

//permet de tirer une balle (sortie puis rentre du doigt)
void Servo_Tir(void)
{
	Servo_Canon_Commande(RC_CANON_ACTIF);
	Delay10KTCYx(50);
	Servo_Canon_Commande(RC_CANON_REPOS);	
}

//met le servo barriere en position basse (les balles peuvent passer)
void OuvrirBarriere(void)
{
	Servo_Barriere_Commande(SERVO_BARRIERE_OUVERT);
}

//met le servo barriere en position haute (les balles sont bloques)
void FermerBarriere(void)
{
	Servo_Barriere_Commande(SERVO_BARRIERE_FERMER);
}

*/



/*************************fonction de commande des servo******************************/

void Servo_Depose_Commande (int Cmd)
{
	if (Cmd!=PWM_Servo_Eject)					// Ne fait la modif que si la valeur est diffrente pour viter d'interdir les interruption
	{
		INTCONbits.GIE=0;						// Interdit les interruptions pendant la copie
		PWM_Servo_Eject = Cmd;					// Met la commande  jour
		INTCONbits.GIE=1;						// Rautorise les interruption
	}
}

void Servo_Canon_Commande (int Cmd)
{
	if (Cmd!=PWM_Servo_Canon)					// Ne fait la modif que si la valeur est diffrente pour viter d'interdir les interruption
	{
		INTCONbits.GIE=0;						// Interdit les interruptions pendant la copie
		PWM_Servo_Canon = Cmd;					// Met la commande  jour
		INTCONbits.GIE=1;						// Rautorise les interruption	
	}
}

void Servo_Barriere_Commande(int Cmd)
{
	if (Cmd!=PWM_Servo_Barriere)				// Ne fait la modif que si la valeur est diffrente pour viter d'interdir les interruption
	{
		INTCONbits.GIE=0;						// Interdit les interruptions pendant la copie
		PWM_Servo_Barriere = Cmd;				// Met la commande  jour
		INTCONbits.GIE=1;						// Rautorise les interruption	
	}
}

void Servo_Option_Commande(int Cmd)
{
	if (Cmd!=PWM_Servo_Option)					// Ne fait la modif que si la valeur est diffrente pour viter d'interdir les interruption
	{
		INTCONbits.GIE=0;						// Interdit les interruptions pendant la copie
		PWM_Servo_Option = Cmd;					// Met la commande  jour
		INTCONbits.GIE=1;						// Rautorise les interruption	
	}
}




/************************************ Vecteur d'interruption Bas ***********************************/

#pragma code InterruptVectorLow = 0x08   // Vecteur d'interruption
void InterruptVectorLow(void)  
{	
	_asm
		goto ServiceInterruptLow	//Saut  la routine d'interruption basse
	_endasm
}
#pragma code


/************************************ Interruption **********************************/
#pragma interruptlow ServiceInterruptLow
void ServiceInterruptLow(void)
{
	if(PIR1bits.TMR1IF) //Dbordement du timer1
	{
		PIR1bits.TMR1IF = 0;		//Acquitement Timer1
		StateMachineServo();    	//Machine  tat des servo
	}
}
#pragma code








/************************************Machine  tat des servo***************************************/
                                                                                              

//Gestion de la machine  tat des servomoteurs
void StateMachineServo()  
{
	static char state = 12;  //Etat de copie des consignes
	static int Copie_PWM_Servo_Eject;
	static int Copie_PWM_Servo_Canon;
	static int Copie_PWM_Servo_Barriere;
	static int Copie_PWM_Servo_Option;
/*
	state = 0	: servo eject : phase  1
	state = 1 	: servo eject : phase  1 variable
 	state = 2 	: servo eject : phase  0 variable

	state = 3	: servo canon : phase  1
	state = 4 	: servo canon : phase  1 variable
 	state = 5 	: servo canon : phase  0 variable

	state = 6	: servo gauche : phase  1
	state = 7 	: servo gauche : phase  1 variable
 	state = 8 	: servo gauche : phase  0 variable

	state = 9	: servo droite : phase  1
	state = 10 	: servo droite : phase  1 variable
 	state = 11 	: servo droite : phase  0 variable

	state = 12  : phase  vide de 10ms pour complter  20ms et copie des consignes
*/
	switch(state)
	{
		/*********** Servo Eject ************/
		//Phase 0
		case 0: WriteTimer1(63035);	//Rechargement du timer de 0,5ms
				SERVO_EJECT=1;
				state = 1;
				break;
		//Phase 1
		case 1: WriteTimer1(65535-Copie_PWM_Servo_Eject);	//Rechargement du timer de 0  2ms
				state = 2;
				break;
		//Phase 2
		case 2: WriteTimer1(55535+Copie_PWM_Servo_Eject);	//Complment  2ms de la valeur prcdente
				SERVO_EJECT=0;
				state = 3;
				break;

		/*********** Servo Canon ************/
		//Phase 3
		case 3: WriteTimer1(63035);	//Rechargement du timer de 0,5ms
				SERVO_CANON=1;
				state = 4;
				break;
		//Phase 4
		case 4: WriteTimer1(65535-Copie_PWM_Servo_Canon);	//Rechargement du timer de 0  2ms
				state = 5;
				break;
		//Phase 5
		case 5: WriteTimer1(55535+Copie_PWM_Servo_Canon);	//Complment  2ms de la valeur prcdente
				SERVO_CANON=0;
				state = 6;
				break;

		/*********** Servo Barriere************/
		//Phase 6
		case 6: WriteTimer1(63035);	//Rechargement du timer de 0,5ms
				SERVO_BARRIERE=1;
				state = 7;
				break;
		//Phase 7
		case 7: WriteTimer1(65535-Copie_PWM_Servo_Barriere);	//Rechargement du timer de 0  2ms
				state = 8;
				break;
		//Phase 8
		case 8: WriteTimer1(55535+Copie_PWM_Servo_Barriere);	//Complment  2ms de la valeur prcdente
				SERVO_BARRIERE=0;
				state = 9;				
				break;

		/*********** Servo Option ************/
		//Phase 9
		case 9: WriteTimer1(63035);	//Rechargement du timer de 0,5ms
				SERVO_OPTION=1;
				state = 10;
				break;
		//Phase 10
		case 10:WriteTimer1(65535-Copie_PWM_Servo_Option);	//Rechargement du timer de 0  2ms 
				state = 11;
				break;
		//Phase 11
		case 11: WriteTimer1(55535+Copie_PWM_Servo_Option);	//Complment  2ms de la valeur prcdente
				SERVO_OPTION=0;
				state = 12;
				break;


		case 12: WriteTimer1(/*15535*/17535);	//Rechargement du timer de 10ms (/*valeur thorique*/valeur rele)
				Copie_PWM_Servo_Eject=PWM_Servo_Eject;
				Copie_PWM_Servo_Canon=PWM_Servo_Canon;
				Copie_PWM_Servo_Barriere=PWM_Servo_Barriere;
				Copie_PWM_Servo_Option=PWM_Servo_Option;
				
				state=0;
				break;
	}

}

