// Fichier Servos.h
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *              En-tete du module de servo              *
 *                                                      *
 * .Date de cration : 19/12/07                         *
 * .Auteur : Jrmy Gardais                             *       
 *                                                      *
 * .Descriptions :                                      *
 *      - Librairie qui gre les 4 servos moteurs de    *
 * la carte barillet                                    *
 *                                                      *
 * .Grandes modifications :                             *
 *  __________________________________________________  *
 * |   DATE   |   Auteur   |          Detail          | *
 * |--------------------------------------------------| *
 * | 19/12/07 | J.Gardais  | Cration et code de la   | *
 * |          |&P.Lucidarme| fonctions: Servo_Init    | *
 * |          |            | InterruptVectorLow       | * 
 * |          |            | ServiceInterruptLow      | *
 * |          |            | StateMachineServo        | *
 * |--------------------------------------------------| *
 * | 23/01/08 |M.Cardineau | -ajout de la config du   | *
 * |          |            |timer1 et des ports       | *
 * | 24/01/08 |            | -Servo_Depose()          | *
 * |          |            | -Servo_Tir()             | * 
 * |          |            | -Interuption_Servomoteur() *
 * |--------------------------------------------------| *
 * | 18/02/08 |M.Cardineau | -OuvrirBarriere()        | *
 * |          |            | -FermerBarriere()        | *
 * |--------------------------------------------------| *
 * |          |            |                          | *
 * |--------------------------------------------------| *
 * |__________|____________|__________________________| *
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *      - Commenter!                                    *
 *      - La librairie utilise le timer1 pour commander *
 * les servomoteurs                                     *
 *      - 0 dans la commande d'un servomoteur			*
 *                                                      *
 ********************************************************/

#ifndef __servo__			// pour viter plusieur appels   la fonction
#define __servo__

//--------------------------DEFINITIONS-----------------------------------------------//

#define SERVO_EJECT 		PORTCbits.RC0			//Port de sortie du servo ejecteur	
#define SERVO_CANON 		PORTCbits.RC1			//Port de sortie du servo de tir
#define SERVO_BARRIERE 		PORTCbits.RC3			//Port de sortie du servo droit
#define SERVO_OPTION 		PORTCbits.RC2			//Port de sortie du servo gauche

#define TRIS_SERVO_EJECT	TRISCbits.TRISC0		//Pour configurer les Servomoteurs
#define TRIS_SERVO_CANON 	TRISCbits.TRISC1
#define TRIS_SERVO_BARRIERE	TRISCbits.TRISC3
#define TRIS_SERVO_OPTION 	TRISCbits.TRISC2



//------------------------------------------------------------------------------------//

/*************************fonction de commande des servo******************************/
void Servo_Init(void);					//Initialisation des servos

void Servo_Depose_Commande (int Cmd);	//
void Servo_Canon_Commande (int Cmd);	//---------------------------------------------------
void Servo_Barriere_Commande(int Cmd);	//permet de donner une consigne de position aux servos
void Servo_Option_Commande(int Cmd);	//---------------------------------------------------


void InterruptVectorLow(void);			// Vecteur d'interruption
void ServiceInterruptLow(void);			// Spooller pour rediriger l'interruption



//Fonction Interuption_Servomoteur
/*Fonction pour gerer la commande des servosmoteur en fonction du Timer1
 *Utilise une machine a tat de 12 tat (3 par servos)
 *On commande alternativement les 4 servos en 2.5ms chacun puis on laisse 10ms de temps mort
 */ 

void StateMachineServo(void);  	//Gestion de la machine  tat des servomoteurs
//------------------------------------------------------------------------------------//

#endif __servo__
