// Fichier main.c
/////////////////////////////////////////////////////////

/****************************************************************
 *                                                    			*
 * .Date de cration : 11/01/08                         		*
 * .Auteur : P. Lucidarme										*
 *							                              		*
 * .Descriptions :                                      		*
 *      - Fichier principal pour la gestion des servo 			*
 *      du barillet   											*
 *                                                      		*
 *                                                      		*
 *                                                      		*
 * .Remarques :                                         		*
 *																*
 ****************************************************************/


// ------------ Directives de compilation
#pragma config WDT = OFF	// Pas de  watchdog
#pragma config OSC = HS		// Quartz HS pour un quartz 20MHz
#pragma config LVP = OFF	// Low voltage programming disabled

// --------------------------------------------------------------------------
// Enttes ncessaires

#include <p18f252.h>		// Pour les registres du PIC18F252
#include <adc.h>			// Pour la config des convertisseurs Analogiques / Numriques
#include <delays.h>			// Pour les tempos et autres time out

#include "Servos.h"			// Librairie pour les calcul de trigo (spcialement l'atan2)

#define					LED_ROUGE				PORTAbits.RA0			// Led rouge
#define					TRIS_LED_ROUGE			TRISAbits.TRISA0		
#define					SWITCH_DEPOSE			PORTBbits.RB5			// Switch de l'jecteur, actif  l'tat haut
#define					TRIS_SWITCH_DEPOSE		TRISBbits.TRISB5
#define					SWITCH_CANON			PORTBbits.RB4			// Switch du canon, actif  l'tat haut
#define					TRIS_SWITCH_CANON  		TRISBbits.TRISB4

#define					CMD_BARRIERE			PORTBbits.RB3
#define					CMD_DEPOSE				PORTBbits.RB0
#define					CMD_CANON				PORTBbits.RB1
#define					CMD_OPTION				PORTBbits.RB2
#define					TRIS_CMD_BARRIERE		TRISBbits.TRISB3
#define					TRIS_CMD_DEPOSE			TRISBbits.TRISB0
#define					TRIS_CMD_CANON			TRISBbits.TRISB1
#define					TRIS_CMD_OPTION			TRISBbits.TRISB2


#define 				RC_DEPOSE_REPOS			7880		// ?? Diminuer pour approcher de la bute
#define 				RC_DEPOSE_ACTIF			5350		// ?? Augmenter pour pousser la balle plus loin

#define 				RC_CANON_REPOS			3200		//Augmenter pour approcher de la bute (5500 pour la mthode 1)
#define 				RC_CANON_ACTIF			5700		//Diminuer pour pousser la balle plus loin

#define 				SERVO_BARRIERE_OUVERT	6000		//position basse (les balles passent)
#define 				SERVO_BARRIERE_FERMER	3000		//position haute (les balles bloques)

#define					SERVO_GUIDE_DROITE		2200		// Position pour envoyer les balles vers la droite
#define					SERVO_GUIDE_GAUCHE		6750		// Position pour envoyer les balles vers la gauche


// Fonction qui fait clignoter la led rouge pour indiquer qu'il y a une erreur
void Error(void);			


void main()
{
	
	int RC_Depose_Calibre=RC_DEPOSE_REPOS;				// Valeur calibre du servo de dpose
	int RC_Canon_Calibre=RC_CANON_REPOS;					// Valeur calibre du servo du canon
	
	OpenADC (ADC_0ANA_0REF,ADC_INT_OFF);				// Tout en numrique
	TRIS_SWITCH_DEPOSE=1;								// Switch de l'ejecteur en entre
	TRIS_SWITCH_CANON=1;								// Switch du canon en entre
	TRIS_LED_ROUGE=0;									// Config de la led rouge							
	TRIS_CMD_BARRIERE=1;								// Config des entre de commande
	TRIS_CMD_DEPOSE=1;
	TRIS_CMD_CANON=1;	
	TRIS_CMD_OPTION=1;										
	
	Servo_Init();										// Initialise les servomoteurs
	LED_ROUGE=0;										// Allume la led rouge pour marquer le dbut de la calibration
	INTCONbits.PEIE=1;									// Autorise les interruptions priphriques
	INTCONbits.GIE=1;									// Autorisation gnrale des interruptions
	
	LED_ROUGE=1;										// Eteind la led rouge pour marquer la fin de la calibration	

	Servo_Canon_Commande (RC_CANON_REPOS);
	Servo_Depose_Commande (RC_DEPOSE_REPOS	);


	// Boucle principale
	
	while (1)											// Scrute les ports de commande pour activer ou non les servos
	{
		if (CMD_DEPOSE)		
			Servo_Depose_Commande (RC_DEPOSE_ACTIF);
		else		
			Servo_Depose_Commande (RC_DEPOSE_REPOS	);
		
		
		
		if (CMD_CANON)
			Servo_Canon_Commande (RC_CANON_ACTIF);
		else
			Servo_Canon_Commande (RC_CANON_REPOS);
	
	
	
		if (CMD_BARRIERE)
			Servo_Barriere_Commande(SERVO_BARRIERE_FERMER);
		else
			Servo_Barriere_Commande(SERVO_BARRIERE_OUVERT);
			
			
		
		
		if (CMD_OPTION)
			Servo_Option_Commande (SERVO_GUIDE_GAUCHE);			// Place le servo pour une dpose  gauche
		else
			Servo_Option_Commande (SERVO_GUIDE_DROITE);			// Place le servo pour une dpose  droite
		
	}
}	




// Fonction qui fait clignoter la led rouge pour indiquer qu'il y a une erreur
void Error(void)
{
	INTCONbits.GIE=0;	
	while (1)
	{
		LED_ROUGE=!LED_ROUGE;
		Delay10KTCYx(50);
	}
	
}	